/*
 *	datamir.h
 *
 *	This file defines an OPaC mirror class which can be used to access data
 *	which is stored in containers, using complicated extractors.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_DATAMIR_H_
#define	_OPaC_DATAMIR_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (datamir.h);

#ifndef _OPaC_CONTAINER_H_
#include <OPaC/container.h>
#endif

/****************************************************************/

class OP_PUBLIC OPDataMirror : public OPGenericContainer
{
	
	struct DataMirrorCache;

protected:
	OPGenericContainerRef	container;			//	container holding the data

	Bool					x_defined;			//	TRUE => extraction defined
	OPArrayRef				x_args;				//	extraction arguments
	
	Count					index;				//	index to current data

	DataMirrorCache*		cache;				//	last object which has been accessed

protected:
	OPDataMirror ();
	virtual ~ OPDataMirror ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */

public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	//	The data mirror reflects an object stored in some container. It has to
	//	be defined through following parameters :
	//
	//	 - The container itself.
	//	 - The extraction (name and arguments).
	//	 - The index.
	//
	//	Usually, the container and the extraction are specified once and then
	//	the index is specified once or several times...
	
	virtual void SetContainer (OPGenericContainerIn source);
	virtual void SetIndex (Count index);
	
	virtual void FlushCache () const;
	virtual void EnableCache (Bool mode = TRUE) const;
	
	virtual Bool GetObject (OPObjectOut object) const;
	
	//	Since the Data Mirror also mimics a generic container, it has to override
	//	following methods :
	
	virtual Count TotalCount () const;					/* OVERRIDDEN */
	virtual Bool Get (Count index, OPObjectOut object);	/* OVERRIDDEN */
	virtual Bool SpecifyExtraction (OPArrayIn x);		/* OVERRIDDEN */
	virtual void SpecifyNoExtraction ();
	
	//	All the following methods override the ones provided by class OPObject
	//	and refer to the mirrored object.
	
	virtual Count CountPublicData () const;
	
	virtual Bool GetPublicData (Count index, OPAtomOut name, OPStringOut desc, OPTypeInfoOut type) const;
	virtual Bool GetPublicDataNotifier (OPAtomIn name, Count nth_notifier, OPAtomOut notifier) const;
	
	virtual Bool QueryData (OPAtomIn name, OPObjectOut data) const;
	virtual Bool QueryData (OPAtomIn name, OPValueOut data) const;
	virtual Bool QueryData (Count index, OPObjectOut data) const;
	virtual Bool QueryData (Count index, OPValueOut data) const;
	
	virtual Bool DefineData (OPAtomIn name, OPObjectIn data);
	virtual Bool DefineData (OPAtomIn name, OPValueIn data);
	virtual Bool DefineData (Count index, OPObjectIn data);
	virtual Bool DefineData (Count index, OPValueIn data);
	
	OP_DEFINE_CLASS (OPDataMirror, OPGenericContainer);
};

/****************************************************************/

#endif /* _OPaC_DATAMIR_H_ */

