/*
 *	dialog.h
 *
 *	This file defines the class used to handle dialogs within OPaC.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_DIALOG_H_
#define	_OPaC_DIALOG_H_

#ifndef _OPaC_DATASLOT_H_
#include <OPaC/dataslot.h>
#endif

OP_INTERFACE (dialog.h);

/****************************************************************/

class OP_PUBLIC OPDialog : public OPDataSlot
{
	
	static OPDialogRef	head;				//	head of active dialog list
	static OPDialogRef	tail;				//	tail of active dialog list
	
private:
	OPDialogRef			next;				//	next active dialog
	OPDialogRef			prev;				//	previous active dialog

	OPDataSlotRef		slot;				//	dialog data slot
	OPWindowRef			window;				//	dialog window
	OPAtomRef			name;				//	dialog name
	
protected:
	OPDialog ();
	virtual ~ OPDialog ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool GetWindow (OPWindowOut window) const;
	virtual Bool GetName (OPAtomOut name) const;
	
	virtual Bool Interaction (OPAtomIn name);
	
	OP_DEFINE_CLASS (OPDialog, OPDataSlot);
};

/****************************************************************/

#endif /* _OPaC_DIALOG_H_ */

