/*
 *	dragpal.h
 *
 *	The drag palette is a view from which other views can be dragged. These
 *	automatically get cloned and stored into a "drag view".
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_DRAGPAL_H_
#define	_OPaC_DRAGPAL_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (dragpal.h);

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

/****************************************************************/

/*
 *	The drag palette exports its subviews in such a way
 *	that they can be torn off, dragged and dropped to
 *	some other place.
 */

class OP_PUBLIC OPDragPalette : public OPControl
{
	
protected:
	Bool				auto_copy;				//	TRUE => automatically copy objects
	OPViewRef			dragging;				//	view being dragged
	
protected:
	OPDragPalette ();
	virtual ~ OPDragPalette ();

	virtual void StoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);		/* OVERRIDDEN */

public:
	virtual void Initialise (OPViewIn view);
	
	virtual void SetAutoCopy (Bool copy);
	virtual Bool ReturnAutoCopy () const;
	
	virtual void NotifyCommitDrop ();
	virtual void NotifyCancelDrop ();

	virtual void Delete ();									/* OVERRIDDEN */
	virtual Bool WouldAcceptChild (OPViewIn child) const;	/* OVERRIDDEN */
	virtual void NotifyInsertChild (OPViewIn child);		/* OVERRIDDEN */
	virtual void NotifyRemoveChild (OPViewIn child);		/* OVERRIDDEN */
	virtual void ButtonPress (OPEventIn event);				/* OVERRIDDEN */
	
	virtual Bool DragAndDrop (OPRectIn, OPObjectIn, OPDragDropProtocol); /* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPDragPalette, OPControl);
};

/****************************************************************/

#endif /* _OPaC_DRAGPAL_H_ */
