/*
 *	freetype.h
 *
 *	Defines the interface to the FreeType library.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_FREETYPE_H_
#define	_OPaC_FREETYPE_H_

#include <OPaC/types.h>
#include <OPaC/opexport.h>

/****************************************************************/

#if defined(WIN32)
	#ifdef __BUILD_FREETYPE_DLL__
		#define	TT_PUBLIC		__declspec(dllexport)
	#else
		#define TT_PUBLIC		__declspec(dllimport)
	#endif
#else
	#define	TT_PUBLIC
#endif

#define	TT_EXPORT(c,s)				OP_EXPORT_CLASS(c);  TT_DEFINE_REF(c,s)
#define	TT_DEFINE_REF(c,s)			OP_DEFINE_X_REF(c,s,TT_PUBLIC)
#define	TT_DEFINE_REF_BEGIN(c,s)	OP_DEFINE_X_REF_BEGIN(c,s,TT_PUBLIC)
#define	TT_DEFINE_REF_END(c,s)		OP_DEFINE_X_REF_END(c,s,TT_PUBLIC)
#define	TT_DEFINE_ARRAY(c,s)		OP_DEFINE_X_ARRAY(c,s,TT_PUBLIC)

/****************************************************************/

TT_EXPORT (OPFreeType, OPObject);				//	<OPaC/freetype.h>
TT_DEFINE_ARRAY (OPFreeType, OPObject);			//	<OPaC/freetype.h>

/****************************************************************/

struct OPaC_TTGlyph
{
	Card8*		bitmap;							//	pointer to bitmap (if any)
	Card8*		pixmap;							//	pointer to pixmap (if any)
	
	Card16		dx;								//	bitmap/pixmap width in pixels
	Card16		dy;								//	bitmap/pixmap height in pixels
	Card16		b_width;						//	bitmap width in bytes
	Card16		p_width;						//	pixmap width in bytes
	
	Card16		x_bearing;						//	left bearing
	Card16		x_advance;						//	horizontal advance
	
	Card16		x_min;							//	leftmost co-ordinate
	Card16		x_max;							//	rightmost co-ordinate
	Card16		y_min;							//	bottom co-ordinate
	Card16		y_max;							//	top co-ordinate
};

/****************************************************************/

struct OPaC_FreeType;

class TT_PUBLIC OPFreeType : public OPObject
{
	
private:
	OPaC_FreeType*		tt;
	OPaC_TTGlyph*		glyphs;
	Count				num_glyphs;
	int					size;
	Bool				is_open;
	
protected:
	OPFreeType ();
	virtual ~ OPFreeType ();
	
public:
	virtual Bool OpenFace (const char* font_name, int size);
	virtual Bool CloseFace ();
	
	virtual Bool GetMonoBitmap (UnicodeChar index, const OPaC_TTGlyph*& pixels);
	virtual Bool GetGrayBitmap (UnicodeChar index, const OPaC_TTGlyph*& pixels);
	
	OP_DEFINE_CLASS (OPFreeType, OPObject);
};

/****************************************************************/

#endif /* _OPaC_FREETYPE_H_ */
