/*
 *	icostyle.h
 *
 *	Defines the icon styles.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_ICOSTYLE_H_
#define	_OPaC_ICOSTYLE_H_

/****************************************************************/

/*
 *	The OPaC look manager provides a collection of icons
 *	which are commonly used for the user interface. They
 *	can be accessed using the following indexes :
 */

enum OPLookIcon
{
	OP_ICON_NONE			= -1,				//	no icon at all
	
	OP_ICON_UP_NORMAL		= 0,				//	arrow up: normal icon
	OP_ICON_UP_PRESS		= 1,				//	arrow up: pressed icon
	OP_ICON_UP_HILITE		= 2,				//	arrow up: highlighted icon
	OP_ICON_UP_DIM			= 3,				//	arrow up: dimmed icon
	
	OP_ICON_DOWN_NORMAL		= 4,				//	arrow down: normal icon
	OP_ICON_DOWN_PRESS		= 5,				//	arrow down: pressed icon
	OP_ICON_DOWN_HILITE		= 6,				//	arrow down: highlighted icon
	OP_ICON_DOWN_DIM		= 7,				//	arrow down: dimmed icon
	
	OP_ICON_LEFT_NORMAL		= 8,				//	arrow left: normal icon
	OP_ICON_LEFT_PRESS		= 9,				//	arrow left: pressed icon
	OP_ICON_LEFT_HILITE		= 10,				//	arrow left: highlighted icon
	OP_ICON_LEFT_DIM		= 11,				//	arrow left: dimmed icon
	
	OP_ICON_RIGHT_NORMAL	= 12,				//	arrow right: normal icon
	OP_ICON_RIGHT_PRESS		= 13,				//	arrow right: pressed icon
	OP_ICON_RIGHT_HILITE	= 14,				//	arrow right: highlighted icon
	OP_ICON_RIGHT_DIM		= 15,				//	arrow right: dimmed icon
	
	OP_ICON_ACCEPT_NORMAL	= 16,				//	"OK" pictogram: normal icon
	OP_ICON_ACCEPT_PRESS	= 17,				//	"OK" pictogram: pressed icon
	OP_ICON_ACCEPT_HILITE	= 18,				//	"OK" pictogram: highlighted icon
	OP_ICON_ACCEPT_DIM		= 19,				//	"OK" pictogram: dimmed icon
	
	OP_ICON_CANCEL_NORMAL	= 20,				//	"ESC" pictogram: normal icon
	OP_ICON_CANCEL_PRESS	= 21,				//	"ESC" pictogram: pressed icon
	OP_ICON_CANCEL_HILITE	= 22,				//	"ESC" pictogram: highlighted icon
	OP_ICON_CANCEL_DIM		= 23,				//	"ESC" pictogram: dimmed icon
	
	OP_ICON_RADIO_0_NORMAL	= 24,				//	radio button: off & normal
	OP_ICON_RADIO_1_NORMAL	= 25,				//	radio button: on & normal
	OP_ICON_RADIO_0_PRESS	= 26,				//	radio button: off & pressed
	OP_ICON_RADIO_1_PRESS	= 27,				//	radio button: on & pressed
	OP_ICON_RADIO_0_HILITE	= 28,				//	radio button: off & highlighted
	OP_ICON_RADIO_1_HILITE	= 29,				//	radio button: on & highlighted
	OP_ICON_RADIO_0_DIM		= 30,				//	radio button: off & dimmed
	OP_ICON_RADIO_1_DIM		= 31,				//	radio button: on & dimmed
	
	OP_ICON_CHECK_0_NORMAL	= 32,				//	check box: off & normal
	OP_ICON_CHECK_1_NORMAL	= 33,				//	check box: on & normal
	OP_ICON_CHECK_0_PRESS	= 34,				//	check box: off & pressed
	OP_ICON_CHECK_1_PRESS	= 35,				//	check box: on & pressed
	OP_ICON_CHECK_0_HILITE	= 36,				//	check box: off & highlighted
	OP_ICON_CHECK_1_HILITE	= 37,				//	check box: on & highlighted
	OP_ICON_CHECK_0_DIM		= 38,				//	check box: off & dimmed
	OP_ICON_CHECK_1_DIM		= 39,				//	check box: on & dimmed

	OP_ICON_LINE_0			= 39,
	
	OP_ICON_I_BEAM			= 0x1000+0,			//	I-beam
	OP_ICON_I_BEAM_MASK		= 0x1000+1,			//	I-beam mask used for the cursor
	
	OP_ICON_GREEN_CLED_OFF	= 0x1000+2,			//	circular green LED: off icon
	OP_ICON_GREEN_CLED_ON	= 0x1000+3,			//	circular green LED: on icon (100%)
	OP_ICON_GREEN_CLED_HALF	= 0x1000+4,			//	circular green LED: on icon (50%)
	
	OP_ICON_RED_CLED_OFF	= 0x1000+5,			//	circular red LED: off icon
	OP_ICON_RED_CLED_ON		= 0x1000+6,			//	circular red LED: on icon (100%)
	OP_ICON_RED_CLED_HALF	= 0x1000+7,			//	circular red LED: on icon (50%)
	
	OP_ICON_GREEN_SLED_OFF	= 0x1000+8,			//	square green LED: off icon
	OP_ICON_GREEN_SLED_ON	= 0x1000+9,			//	square green LED: on icon (100%)
	OP_ICON_GREEN_SLED_HALF	= 0x1000+10,		//	square green LED: on icon (50%)
	
	OP_ICON_RED_SLED_OFF	= 0x1000+11,		//	square red LED: off icon
	OP_ICON_RED_SLED_ON		= 0x1000+12,		//	square red LED: on icon (100%)
	OP_ICON_RED_SLED_HALF	= 0x1000+13,		//	square red LED: on icon (50%)

	OP_ICON_H_SLIDER_LEFT	= 0x1000+14,		//	horizontal slider handle: left half
	OP_ICON_H_SLIDER_RIGHT	= 0x1000+15,		//	horizontal slider handle: right half
	OP_ICON_H_SLIDER_SMALL	= 0x1000+16,		//	small horizontal slider handle
	
	OP_ICON_V_SLIDER_TOP	= 0x1000+17,		//	vertical slider handle: left half
	OP_ICON_V_SLIDER_BOT	= 0x1000+18,		//	vertical slider handle: right half
	OP_ICON_V_SLIDER_SMALL	= 0x1000+19,		//	small vertical slider handle
	
	OP_ICON_SUBMENU_NORMAL	= 0x1000+20,		//	submenu pictogram: normal icon
	OP_ICON_SUBMENU_PRESS	= 0x1000+21,		//	submenu pictogram: pressed icon
	OP_ICON_SUBMENU_HILITE	= 0x1000+22,		//	submenu pictogram: highlighted icon
	OP_ICON_SUBMENU_DIM		= 0x1000+23,		//	submenu pictogram: dimmed icon
	
	OP_ICON_PULLDOWN_NORMAL	= 0x1000+24,		//	pull-down pictogram: normal icon
	OP_ICON_PULLDOWN_PRESS	= 0x1000+25,		//	pull-down pictogram: pressed icon
	OP_ICON_PULLDOWN_HILITE	= 0x1000+26,		//	pull-down pictogram: highlighted icon
	OP_ICON_PULLDOWN_DIM	= 0x1000+27,		//	pull-down pictogram: dimmed icon
	
	OP_ICON_POPUP_NORMAL	= 0x1000+28,		//	pop-up pictogram: normal icon
	OP_ICON_POPUP_PRESS		= 0x1000+29,		//	pop-up pictogram: pressed icon
	OP_ICON_POPUP_HILITE	= 0x1000+30,		//	pop-up pictogram: highlighted icon
	OP_ICON_POPUP_DIM		= 0x1000+31,		//	pop-up pictogram: dimmed icon
	
	OP_ICON_DOWN_ARROW		= 0x1000+32,		//	large arrow pointing down
	OP_ICON_UP_ARROW		= 0x1000+33,		//	large arrow pointing up
	
	OP_ICON_LINE_1			= 0x1000+33,
	
	OP_ICON_TREE_BRANCH		= 0x2000+0,			//	tree: new branch to the right
	OP_ICON_TREE_TRUNK		= 0x2000+1,			//	tree: no branch, just the trunk
	OP_ICON_TREE_BOTTOM		= 0x2000+2,			//	tree: last branch, bottom of tree
	OP_ICON_TREE_BOX		= 0x2000+3,			//	tree: empty box, branch
	OP_ICON_TREE_BOX_BOT	= 0x2000+4,			//	tree: empty box, bottom of tree
	OP_ICON_TREE_MINUS		= 0x2000+5,			//	tree: minus box, branch
	OP_ICON_TREE_MINUS_BOT	= 0x2000+6,			//	tree: minus box, bottom of tree
	OP_ICON_TREE_PLUS		= 0x2000+7,			//	tree: plus box, branch
	OP_ICON_TREE_PLUS_BOT	= 0x2000+8,			//	tree: plus box, bottom of tree
	OP_ICON_TREE_HOME		= 0x2000+9,			//	tree: home
	OP_ICON_TREE_DIR		= 0x2000+10,		//	tree: directory
	OP_ICON_TREE_DIR_OPEN	= 0x2000+11,		//	tree: open directory
	OP_ICON_TREE_FILE		= 0x2000+12,		//	tree: file
	
	OP_ICON_LINE_2			= 0x2000+12,
	
	OP_ICON_A_PLUS_NORMAL	= 0x3000+0,			//	plus arrow: normal icon
	OP_ICON_A_PLUS_PRESS	= 0x3000+1,			//	plus arrow: pressed icon
	OP_ICON_A_PLUS_HILITE	= 0x3000+2,			//	plus arrow: highlighted icon
	OP_ICON_A_PLUS_DIM		= 0x3000+3,			//	plus arrow: dimmed icon
	
	OP_ICON_A_MINUS_NORMAL	= 0x3000+4,			//	minus arrow: normal icon
	OP_ICON_A_MINUS_PRESS	= 0x3000+5,			//	minus arrow: pressed icon
	OP_ICON_A_MINUS_HILITE	= 0x3000+6,			//	minus arrow: highlighted icon
	OP_ICON_A_MINUS_DIM		= 0x3000+7,			//	minus arrow: dimmed icon
	
	OP_ICON_MINCLOSE_NORMAL	= 0x3000+8,			//	mini close button: normal icon
	OP_ICON_MINCLOSE_PRESS	= 0x3000+9,			//	mini close button: pressed icon
	OP_ICON_MINCLOSE_HILITE	= 0x3000+10,		//	mini close button: highlighted icon
	OP_ICON_MINCLOSE_DIM	= 0x3000+11,		//	mini close button: dimmed icon
	
	OP_ICON_SLIDOT_NORMAL	= 0x3000+12,		//	slider dot: normal icon
	OP_ICON_SLIDOT_PRESS	= 0x3000+13,		//	slider dot: pressed icon
	OP_ICON_SLIDOT_HILITE	= 0x3000+14,		//	slider dot: highlighted icon
	OP_ICON_SLIDOT_DIM		= 0x3000+15,		//	slider dot: dimmed icon
	
	OP_ICON_LINE_3			= 0x3000+15,
	
	OP_ICON_CUBE_NORMAL		= 0x4000+0,			//	cube: icon for a normal object
	OP_ICON_CUBE_NULL		= 0x4000+1,			//	cube: icon for a null object
	OP_ICON_CUBE_HILITE		= 0x4000+2,			//	cube: highlighted icon
	OP_ICON_CUBE_ACTIVE		= 0x4000+3,			//	cube: highlighted icon (2nd version)
	
	OP_ICON_TRASH_NORMAL	= 0x4000+4,			//	trashcan: normal state
	OP_ICON_TRASH_EMPTY		= 0x4000+5,			//	trashcan: empty state
	OP_ICON_TRASH_HILITE	= 0x4000+6,			//	trashcan: highlighted icon
	OP_ICON_TRASH_ACTIVE	= 0x4000+7,			//	trashcan: highlighted icon (2nd version)
	
	OP_ICON_RIGHT_ARROW		= 0x4000+8,			//	large arrow pointing to the right
	OP_ICON_LEFT_ARROW		= 0x4000+9,			//	large arrow pointing to the left
	
	OP_ICON_OBJ_APP			= 0x4000+10,		//	object: application
	OP_ICON_OBJ_WINDOW		= 0x4000+11,		//	object: window
	OP_ICON_OBJ_MENU		= 0x4000+12,		//	object: menu
	OP_ICON_OBJ_SLOT		= 0x4000+13,		//	object: slot
	
	OP_ICON_LINE_4			= 0x4000+13,
	
	OP_ICON_USER			= 0x8000
};

#define	OP_ICON_NUM		( ((Card32)(OP_ICON_LINE_0) & 0x00FF) + 1	\
						+ ((Card32)(OP_ICON_LINE_1) & 0x00FF) + 1	\
						+ ((Card32)(OP_ICON_LINE_2) & 0x00FF) + 1	\
						+ ((Card32)(OP_ICON_LINE_3) & 0x00FF) + 1	\
						+ ((Card32)(OP_ICON_LINE_4) & 0x00FF) + 1 )

OP_TYPEENUM (OPLookIcon);

#endif /* _OPaC_ICOSTYLE_H_ */
