/*
 *	iterator.h
 *
 *	This file exports the generic iterator class which can be
 *	used to iterate over containers, data bases and any other
 *	class derived from the generic container class.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_ITERATOR_H_
#define	_OPaC_ITERATOR_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (iterator.h);

/****************************************************************/

#ifndef _OPaC_CONTAINER_H_
#include <OPaC/container.h>
#endif

/****************************************************************/

/*
 *	Base class for every iterator.
 */

class OP_PUBLIC OPIterator : public OPObject
{
	friend class OPGenericContainer;

protected:
	OPGenericContainerRef	source;			//	data source for this iterator
	Bool					did_change;		//	TRUE => iterator position changed
	Bool					is_valid;		//	TRUE => valid move
	
protected:
	OPIterator ();
	virtual ~ OPIterator ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void SetSource (OPGenericContainerIn source);
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool GetSource (OPGenericContainerOut source) const;
	virtual const OPGenericContainerRef& ReturnSource () const;
	
	virtual Bool DidChange ();
	
	//	The iterator is used to browse data contents. The reference
	//	data base (container/set/array/etc.) must provide a derived
	//	iterator which implements following navigation methods :
	
	virtual Bool First () = 0;
	virtual Bool Last () = 0;
	
	virtual Bool Next (Count move = 1) = 0;
	virtual Bool Prev (Count move = 1) = 0;
	
	//	Data can be retrieved through the Get methods. The derived
	//	classes need only override method "Bool Get (object, index)".
	
	virtual OPObjectRef Get (Count index = 0);
	virtual Bool Get (OPObjectOut object, Count index = 0) = 0;
	
	virtual Bool ForEach (OPPerformerIn performer, Count index = 0);
	
	OP_DEFINE_CLASS (OPIterator, OPObject);
};

/****************************************************************/

#endif /* _OPaC_ITERATOR_H_ */
