/*
 *	menu.h
 *
 *	The Menu class can be used to build independent menus, pop up menus and
 *	pull down menus.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_MENU_H_
#define	_OPaC_MENU_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (menu.h);

#ifndef _OPaC_WINDOW_H_
#include <OPaC/window.h>
#endif

/****************************************************************/

/*
 *	A menu is a special kind of window which has a cell
 *	array associated to it.
 */

class OP_PUBLIC OPMenu : public OPWindow
{
	
protected:
	OPCellGridRef		grid;				//	cell grid
	Count				last_choice;		//	last choice made in menu
	Bool				is_automatic;		//	popped up automatically
	
	Card32				end_buttons;		//	button(s) to wait for
	
protected:
	OPMenu ();
	virtual ~ OPMenu ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Awake ();								/* OVERRIDDEN */
	virtual void AwakeFromApp (OPAppIn app);			/* OVERRIDDEN */
	virtual void AwakeFromObject (OPObjectIn object);	/* OVERRIDDEN */
	
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual void SetCellGrid (OPCellGridIn grid);
	virtual const OPCellGridRef& ReturnCellGrid () const;
	
	virtual void ShowMenu (OPCoord x, OPCoord y);
	virtual void HideMenu ();

	virtual void SetEndButtons (Card32 buttons);
	virtual Card32 ReturnEndButtons () const;
	
	virtual void PopupAsPopup (OPObjectIn button, Card32 id);
	virtual void PopupAsSubmenu (OPObjectIn button, Card32 id);
	virtual void PopupAsPulldown (OPObjectIn button, Card32 id);
	
	virtual void HandleEnterNotification (OPObjectIn cell, Card32 id);
	virtual void HandleLeaveNotification (OPObjectIn cell, Card32 id);
	
	virtual void DispatchEvent (OPEventIn event);		/* OVERRIDDEN */
	virtual void LeaveNotify (OPEventIn event);			/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPMenu, OPWindow);
};

#endif /* _OPaC_MENU_H_ */

