/*
 *	monitorana.h
 *
 *	Analogic data monitor. This is a kind of gauge.
 *
 *	(C) Copyright 1997-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_MONITORANA_H_
#define	_OPaC_MONITORANA_H_

#ifndef _OPaC_MONITORVALUE_H_
#include <OPaC/monitorvalue.h>
#endif

OP_INTERFACE (monitorana.h);

/****************************************************************/

class OP_PUBLIC OPMonitorAnalogic : public OPMonitorValue
{

protected:
	Real				dis_value;			//	value currently displayed by the needle
	Real				max_value;			//	maximum (absolute) value
	
	Bool				is_symmetric;		//	TRUE => 0 in the middle
	Bool				is_rectified;		//	TRUE => take the absolute value
	Bool				is_dynamic_max;		//	TRUE => dynamically define maximum
	
	Bool				negative_flag;		//	TRUE => display "-" flag
	
protected:
	OPMonitorAnalogic ();
	virtual ~ OPMonitorAnalogic ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void DoUpdate ();							/* OVERRIDDEN */
	
public:
	virtual Bool SetMaximum (Real max);
	virtual Bool SetSymmetric (Bool yes);
	virtual Bool SetRectified (Bool yes);
	virtual Bool SetDynamicMax (Bool yes);
	
	Real ReturnMaximum () const				{ return this->max_value; }
	Bool IsSymmetric () const				{ return this->is_symmetric; }
	Bool IsRectified () const				{ return this->is_rectified; }
	Bool IsDynamicMax () const				{ return this->is_dynamic_max; }
	
	//	Overridden methods :
	
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);
	
	OP_DEFINE_CLASS (OPMonitorAnalogic, OPMonitorValue);
};

/****************************************************************/

#endif /* _OPaC_MONITORANA_H_ */

