/*
 *	opacirt.h
 *
 *	This file defines OPaC Internal Run-Time classes, which are of no interest
 *	to the developer.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_OPACIRT_H_
#define	_OPaC_OPACIRT_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_ATOM_H_
#include <OPaC/atom.h>
#endif

#ifndef _OPaC_TARGETMACRO_H_
#include <OPaC/targetmacro.h>
#endif

#ifndef _OPaC_PUBMACRO_H_
#include <OPaC/pubmacro.h>
#endif

OP_INTERFACE (opacirt.h);

/****************************************************************/

/*      
 *	The run-time target publishing class is used internally
 *	during initialization only. Never use this class yourself !
 */

class OPaC_Target
{
	friend class OPRuntime;
	friend class OPObject;
	friend class OPTarget;
	friend class OPMessage;
	
protected:
	OPaC_Target*		next;					//	next target description in same class
	
	OPRuntimeRef		target_class;			//	class for the target
	OPaC_TargetMethod*	target_method;			//	description of target method
	
	OPAtomRef			name;					//	atomic representation of target name
	
protected:
	void Initialise ();

public:
	OPaC_Target (OPRuntimeIn the_class,
				 OPaC_TargetMethod* the_method)
		: target_class (the_class),
		  target_method (the_method),
		  next (0)	{ }
	
	const OPaC_TargetMethod* ReturnTargetMethod () const	{ return this->target_method; }
	const OPAtomRef& ReturnName () const					{ return this->name; }
};

/****************************************************************/

/*      
 *	The run-time data publishing class is used internally
 *	during initialization only. Never use this class yourself !
 */

class OP_PUBLIC OPaC_PubData
{
	friend class OPRuntime;
	friend class OPObject;
	friend class OPBitsetInfo;

	struct BitsetData
	{
		BitsetData*		next;
		const OPaC_BitsetDataDef* def;
		Card32			num;
	};
	
protected:
	OPaC_PubData*		next;					//	next data descrtipiton in same class
	BitsetData*			bitset_list;			//	list of bitset data definitions
	Card32				bitset_num;				//	# bitset data definitions
	OPBitsetInfoRef		bitset_info;			//	compiled bitset information
	
	OPaC_PubDataDef		data_def;				//	description of the published data
	OPaC_PubData*		sim_next;				//	next data definition with same name
	OPAtomRef			name;					//	atomic representation of data name

protected:
	void Initialise () { this->name = this->data_def.var_name; }
	
public:
	OPaC_PubData (OPaC_PubDataDef& def)
		: data_def (def),
		  sim_next (0),
		  bitset_list (0),
		  bitset_num (0),
		  next (0)	{ }
	
	const OPaC_PubDataDef* ReturnPubDataDef () const	{ return & this->data_def; }
	const OPAtomRef& ReturnName () const				{ return this->name; }
	const OPBitsetInfoRef& ReturnBitsetInfo () const	{ return this->bitset_info; }
};

/****************************************************************/

/*
 *	The target definition class is very useful in building
 *	dynamic message forwarding systems, since the receiving
 *	member functions may be linked at run-time.
 */

class OP_PUBLIC OPTarget : public OPObject
{
	
private:
	OPAtomRef			name;					//	target name
	OPObjectRef			object;					//	target object
	OPTargetType		type;					//	target type
	const OPaC_Target*	target;					//	target description (if any)
	
protected:
	OPTarget ();
	virtual ~ OPTarget ();
	
	virtual Bool Attach ();

	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool AttachToObject (OPObjectIn object);
	
	virtual Bool SetTarget (OPAtomIn name, OPObjectIn object, OPTargetTypeIn type);
	virtual void GetTarget (OPAtomOut name, OPObjectOut object, OPTargetTypeOut type) const;
	
	virtual const OPAtomRef& GetName () const;
	virtual const OPObjectRef& GetObject () const;
	virtual OPTargetType GetType () const;
	
	virtual Bool IsValid () const;
	
	virtual Bool DoReport (OPObjectIn object, Card32 value) const;
	virtual Bool DoAction () const;
	virtual Bool DoMessage (OPMessageInOut message) const;
	
	OP_DEFINE_CLASS (OPTarget, OPObject);
};

/****************************************************************/

#endif /* _OPaC_OPACIRT_H_ */
