/*
 *	performer.h
 *
 *	The Performer base class is used to perform repetitive
 *	actions on data sets, based on iterators.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_PERFORMER_H_
#define	_OPaC_PERFORMER_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (performer.h);

/****************************************************************/

/*
 *	Base class for every performer.
 */

class OP_PUBLIC OPPerformer : public OPObject
{

protected:
	
	
protected:
	OPPerformer ();
	virtual ~ OPPerformer ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool Perform (OPObjectIn object) = 0;
	
	OP_DEFINE_CLASS (OPPerformer, OPObject);
};

/****************************************************************/

#define	OP_PERFORMER(n,T,v)										\
																\
class n : public OPPerformer									\
{																\
	T	v;														\
public:															\
	n (T x) : v (x) { }											\
	virtual ~ n () { }											\
	virtual Bool Perform (OPObjectIn object);					\
};																\
																\
Bool															\
n::Perform (OPObjectIn object)

#define	OP_SIMPLE_PERFORMER(n)									\
																\
class n : public OPPerformer									\
{																\
public:															\
	n () { }													\
	virtual ~ n () { }											\
	virtual Bool Perform (OPObjectIn object);					\
};																\
																\
Bool															\
n::Perform (OPObjectIn object)



#endif /* _OPaC_PERFORMER_H_ */
