/*
 *	targetmacro.h
 *
 *	This the macros which will be used to export information about
 *	methods, observers and actors.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_TARGETMACRO_H_
#define	_OPaC_TARGETMACRO_H_

#ifndef _OPaC_OBJECT_H_
#include <OPaC/object.h>
#endif

OP_INTERFACE (targetmacro.h);

/****************************************************************/

/*
 *	OP_TARGET_CLASS (OPScroller);
 *	OP_TARGET_ARG (Real, ratio);
 *	OP_TARGET_CALL (SetRatio (ratio));
 *	OP_TARGET_END ();
 */

#define	OP_TARGET_CLASS(c)										\
	OP_TARGET_CLASS_1(c,__LINE__)

#define	OP_TARGET_CLASS_1(c,l)									\
	OP_TARGET_CLASS_2(c,l)

#define	OP_TARGET_CLASS_2(c,l)									\
																\
Bool trampo_##c##l (OPMessageIn,OPObject*,OPaC_TargetExport*);	\
																\
OPaC_TargetExport auto_##c##l (trampo_##c##l, OP_RUNTIME(c));	\
																\
Bool															\
trampo_##c##l (OPMessageIn _message, OPObject* _object,			\
			   OPaC_TargetExport* _reg_target)					\
{																\
	c* _instance = (c*)(_object)


#define	OP_TARGET_ARG(T,arg,desc)								\
	T arg;														\
	if (_reg_target) {											\
		_reg_target->SetArg (#T, #arg, desc);					\
	} else _message >> arg


#define	OP_TARGET_CALL(call)									\
	if (_reg_target) {											\
		_reg_target->SetCall (#call);							\
		return TRUE;											\
	}															\
	if (_instance) {											\
		_instance->call

#define	OP_TARGET_END()											\
		return TRUE;											\
	}															\
	return FALSE;												\
}																\
																\
class OPObject

/*
 *	OP_TARGET_ACTOR (OPScroller, Draw);
 */

#define	OP_TARGET_ACTOR(c,a)									\
	OP_TARGET_ACTOR_1(c,a,__LINE__)

#define	OP_TARGET_ACTOR_1(c,a,l)								\
	OP_TARGET_ACTOR_2(c,a,l)

#define	OP_TARGET_ACTOR_2(c,a,l)								\
																\
Bool trampo_##c##l (OPObject*);									\
																\
OPaC_TargetExport auto_##c##l (trampo_##c##l,#a,OP_RUNTIME(c));	\
																\
Bool															\
trampo_##c##l (OPObject* _object)								\
{																\
	c* _instance = (c*)(_object);								\
	_instance->a ();											\
	return TRUE;												\
}																\
																\
class OPObject


/*
 *	OP_TARGET_OBSERVER (OPHandler, OnClick);
 */

#define	OP_TARGET_OBSERVER(c,a)									\
	OP_TARGET_OBSERVER_1(c,a,__LINE__)

#define	OP_TARGET_OBSERVER_1(c,a,l)								\
	OP_TARGET_OBSERVER_2(c,a,l)

#define	OP_TARGET_OBSERVER_2(c,a,l)								\
																\
Bool trampo_##c##l (OPObject*, OPObjectIn, Card32);				\
																\
OPaC_TargetExport auto_##c##l (trampo_##c##l,#a,OP_RUNTIME(c));	\
																\
Bool															\
trampo_##c##l (OPObject* _object, OPObjectIn _obj, Card32 _id)	\
{																\
	c* _instance = (c*)(_object);								\
	_instance->a (_obj, _id);									\
	return TRUE;												\
}																\
																\
class OPObject

/****************************************************************/

/*
 *	A target may be either a method (used by OPMessage), an
 *	observer (for notifications by "OPObject::Report") or an
 *	actor (invoked by "OPObject::Action").
 */

OP_TYPEDEF (Card32, OPTargetType);

#define	OP_TARGET_IS_INVALID	0				//	target: not valid
#define	OP_TARGET_IS_METHOD		1				//	target: it's a method
#define	OP_TARGET_IS_OBSERVER	2				//	target: it's an observer
#define	OP_TARGET_IS_ACTOR		3				//	target: it's an actor

/****************************************************************/

class OPaC_TargetExport;

typedef Bool (*OPaC_TrampoFuncM)(OPMessageIn,OPObject*,OPaC_TargetExport*);
typedef Bool (*OPaC_TrampoFuncA)(OPObject*);
typedef Bool (*OPaC_TrampoFuncO)(OPObject*,OPObjectIn,Card32);

/****************************************************************/

/*
 *	Each method may take n parameters, which are described
 *	using some simple parameter description.
 */

struct OP_PUBLIC OPaC_ParamDesc {
   
	const LatinChar* type;					//	parameter: type
	const LatinChar* name;					//	parameter: name
	const LatinChar* desc;					//	parameter: description
	
	OPTypeDef		type_def;				//	type: type definition (if available)
	OPAtomRef		name_atomic;			//	name: atomic representation
};

/*
 *	The target declaration needs some special structures
 *	which won't be used directly by the programmer, but
 *	will be generated by the OP_TARGET_xxx macros.
 */

struct OP_PUBLIC OPaC_TargetMethod
{
	const OPRuntimeRef*	runtime;			//	runtime description
	const LatinChar*	method;				//	method name
	OPaC_TrampoFuncM	m_trampo;			//	trampoline func. for method using OPMessage
	OPaC_TrampoFuncO	o_trampo;			//	trampoline func. for observer
	OPaC_TrampoFuncA	a_trampo;			//	trampoline func. for actor
	Count				num_param;			//	actual # of parameters taken by method
	OPaC_ParamDesc		param[10];			//	parameter description
};

/****************************************************************/

class OP_PUBLIC OPaC_TargetExport
{
	friend class OPRuntime;
	
private:
	static OPaC_TargetExport*	head;
	static OPaC_TargetExport*	tail;
	
protected:
	OPaC_TargetExport*			next;
	OPaC_TargetExport*			prev;
	OPaC_TargetMethod			info;
	
public:
	OPaC_TargetExport (OPaC_TrampoFuncM, OPRuntimeIn);
	OPaC_TargetExport (OPaC_TrampoFuncA, const char*, OPRuntimeIn);
	OPaC_TargetExport (OPaC_TrampoFuncO, const char*, OPRuntimeIn);
	
	void SetArg (const LatinChar* type, const LatinChar* arg, const LatinChar* desc);
	void SetCall (const LatinChar* call);
};

/****************************************************************/

#endif /* _OPaC_TARGETMACRO_H_ */
