/*
 *	textstyle.h
 *
 *	Defines the text and font styles.
 *
 *	(C) Copyright 1994-1997, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_TEXTSTYLE_H_
#define	_OPaC_TEXTSTYLE_H_

/****************************************************************/

/*
 *	Enumeration of the world-wide existing font weights.
 *	If you find another weight denomination, please tell
 *	me ! I took the Adobe denominations and hope them to
 *	be representative.
 *
 *	The numerical weights have been set arbitrarily with
 *	1.0 being the "Roman" weight (as in Times-Roman).
 */

#define	OP_FONT_ULTRA_LIGHT	((float)(0.1))		//	Ultra Light
#define	OP_FONT_THIN		((float)(0.4))		//	Thin
#define	OP_FONT_LIGHT		((float)(0.6))		//	Light, Extra Light
#define	OP_FONT_BOOK		((float)(0.8))		//	Book
#define	OP_FONT_REGULAR		((float)(0.9))		//	Regular, Plain, Normal
#define	OP_FONT_MEDIUM		((float)(1.0))		//	Medium, Roman
#define	OP_FONT_DEMI_BOLD	((float)(1.1))		//	Demi Bold, Demi
#define	OP_FONT_SEMI_BOLD	((float)(1.2))		//	Semi Bold
#define	OP_FONT_BOLD		((float)(1.3))		//	Bold
#define	OP_FONT_EXTRA_BOLD	((float)(1.4))		//	Extra Bold
#define	OP_FONT_HEAVY		((float)(1.5))		//	Heavy, Heavyface, Ultra Bold
#define	OP_FONT_BLACK		((float)(1.6))		//	Black
#define	OP_FONT_FAT			((float)(1.7))		//	Fat, Ultra, Ultra Black
#define	OP_FONT_OBESE		((float)(1.8))		//	Obese, Extra Black

/****************************************************************/

/*
 *	Text can be positioned horizontally and vertical in
 *	many different ways. These are defined by the following
 *	groups of constants :
 */

#define	OP_TEXT_HPOS_LEFT	((Card8)(0x00))		//	>Hello    <
#define	OP_TEXT_HPOS_CENTER	((Card8)(0x01))		//	>  Hello  <
#define	OP_TEXT_HPOS_RIGHT	((Card8)(0x02))		//	>    Hello<
#define	OP_TEXT_HPOS_ALIGN	((Card8)(0x03))		//	>H e l l o<

#define	OP_TEXT_VPOS_BASE	((Card8)(0x00))		//	use baseline as vertical ref.
#define	OP_TEXT_VPOS_TOP	((Card8)(0x10))		//	use top border as vertical ref.
#define	OP_TEXT_VPOS_CENTER	((Card8)(0x20))		//	use text middle as vertical ref.
#define	OP_TEXT_VPOS_BOTTOM	((Card8)(0x30))		//	use bottom border as vertical ref.

#define	OP_TEXT_HPOS_MASK	((Card8)(0x0F))
#define	OP_TEXT_HPOS_SHIFT	(0)
#define	OP_TEXT_VPOS_MASK	((Card8)(0xF0))
#define	OP_TEXT_VPOS_SHIFT	(4)

/****************************************************************/

#endif /* _OPaC_TEXTSTYLE_H_ */
