/*
 *	turbosrc.h
 *
 *	Turbo Search implementation for OPaC. A very fast way to find
 *	patterns in long texts.
 *
 *	(C) Copyright 1997-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_TURBOSEARCH_H_
#define	_OPaC_TURBOSEARCH_H_

#ifndef	_OPaC_OBJECT_H_
#include <OPaC/object.h>
#endif

/****************************************************************/

/*
 *	Turbo Search is a very fast way to find defined patterns in
 *	text strings. It is a refinement of the Quicksearch algorithm
 *	and has been published by Michael Tamm in c't 8/97, pp.292-297.
 */

class OP_PUBLIC OPTurboSearch : public OPObject
{
protected:
	Count			shift[256];
	Count			look_at;
	Count			look_after;
	Card8*			pattern;
	Count			len;
	Count			last;
	
protected:
	OPTurboSearch ();
	virtual ~ OPTurboSearch ();
	
public:
	virtual void Setup (const LatinChar* pattern, Count len);
	virtual Count Search (const LatinChar* text);
	
	OP_DEFINE_CLASS (OPTurboSearch, OPObject);
};

/****************************************************************/

#endif /* _OPaC_TURBOSEARCH_H_ */
