/*
 *	values.h
 *
 *	Encapsulate values (numbers, integers, booleans, reals, etc.)
 *	so that they can be handled as standard OPaC objects.
 *
 *	(C) Copyright 1994-1997, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_VALUES_H_
#define	_OPaC_VALUES_H_

#ifndef _OPaC_OBJECT_H_
#include <OPaC/object.h>
#include <OPaC/opexport.h>
#endif

OP_INTERFACE (values.h);

/****************************************************************/

OP_DEFINE_REF_BEGIN (OPValue, OPObject);
	
//	OPValueRef (OPObjectIn x);
//	OPValueRef (OPValueIn x);
//	OPValueRef (OPValue* x);
	OPValueRef (Card32 x);
	OPValueRef (Int32 x);
	OPValueRef (Bool x);
	OPValueRef (Real x);
	
//	OPValueRef& operator = (OPObjectIn x);
//	OPValueRef& operator = (OPValueIn x);
//	OPValueRef& operator = (OPValue* x);
	OPValueRef& operator = (Card32 x);
	OPValueRef& operator = (Int32 x);
	OPValueRef& operator = (Bool x);
	OPValueRef& operator = (Real x);

	operator Card32 () const;
	operator Int32 () const;
	operator Bool () const;
	operator Real () const;
	
	OPValue* NewValue ();
	
	friend Bool operator == (OPValueIn v, Card32 x);
	friend Bool operator == (OPValueIn v, Int32 x);
	friend Bool operator == (OPValueIn v, Bool x);
	friend Bool operator == (OPValueIn v, Real x);
	
	friend Bool operator == (Card32 x, OPValueIn v);
	friend Bool operator == (Int32 x, OPValueIn v);
	friend Bool operator == (Bool x, OPValueIn v);
	friend Bool operator == (Real x, OPValueIn v);
	
	friend Bool operator != (OPValueIn v, Card32 x);
	friend Bool operator != (OPValueIn v, Int32 x);
	friend Bool operator != (OPValueIn v, Bool x);
	friend Bool operator != (OPValueIn v, Real x);
	
	friend Bool operator != (Card32 x, OPValueIn v);
	friend Bool operator != (Int32 x, OPValueIn v);
	friend Bool operator != (Bool x, OPValueIn v);
	friend Bool operator != (Real x, OPValueIn v);
	
OP_DEFINE_REF_END (OPValue, OPObject);

/****************************************************************/

extern inline Bool operator == (Card32 x, OPValueIn v) { return (v == x) ? TRUE : FALSE; }
extern inline Bool operator == (Int32 x,  OPValueIn v) { return (v == x) ? TRUE : FALSE; }
extern inline Bool operator == (Bool x,   OPValueIn v) { return (v == x) ? TRUE : FALSE; }
extern inline Bool operator == (Real x,   OPValueIn v) { return (v == x) ? TRUE : FALSE; }

extern inline Bool operator != (Card32 x, OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (Int32 x,  OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (Bool x,   OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (Real x,   OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (OPValueIn v, Card32 x) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (OPValueIn v, Int32 x)  { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (OPValueIn v, Bool x)   { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (OPValueIn v, Real x)   { return (v == x) ? FALSE : TRUE; }

/****************************************************************/

/*
 *	Values wrapper class...
 */

class OP_PUBLIC OPValue : public OPObject
{
	
	union OPaC_ValueUnion {
		Card32			the_card32;
		Int32			the_int32;
		Bool			the_bool;
		Real			the_real;
	};
	
protected:
	OPSimpleType		type;				//	type of value
	OPaC_ValueUnion		value;				//	value (depending on the type)
	OPObjectRef			object;				//	object value (if any)
	OPStringRef			auto_format;		//	automatic conversion format
	
protected:
	OPValue ();
	virtual ~OPValue ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */

	virtual void RefreshValue ();
	virtual void ChangedValue ();
	
	virtual Real ToReal () const;
	
public:
	virtual Count ComputeHashValue (Count num) const;	/* OVERRIDDEN */
	virtual int CompareWith (OPObjectIn object) const;	/* OVERRIDDEN */
	
	virtual Bool SetCard32 (Card32 x);
	virtual Bool SetInt32 (Int32 x);
	virtual Bool SetBool (Bool x);
	virtual Bool SetReal (Real x);
	virtual Bool SetObject (OPObjectIn x);
	virtual Bool SetText (OPTextIn x);
	virtual Bool SetArray (OPArrayIn x);

	virtual Card32 ReturnCard32 () const;
	virtual Int32  ReturnInt32 () const;
	virtual Bool   ReturnBool () const;
	virtual Real   ReturnReal () const;
	virtual const OPObjectRef& ReturnObject () const;
	virtual const OPObjectRef& ReturnText () const;
	virtual const OPObjectRef& ReturnArray () const;

	virtual Bool SetAutoFormat (OPStringIn format);
	virtual const OPStringRef& ReturnAutoFormat () const;
	
	virtual Bool ConvertToString (OPStringOut text, OPStringIn format = OPStringNull) const;
	virtual Bool ConvertToAtom (OPAtomOut atom, OPStringIn format = OPStringNull) const;
	
	virtual int CompareWith (Card32 x) const;
	virtual int CompareWith (Int32 x) const;
	virtual int CompareWith (Bool x) const;
	virtual int CompareWith (Real x) const;
	
	virtual OPSimpleType ReturnType () const;
	
	OP_DEFINE_CLASS_SPECIAL_REF (OPValue, OPObject);
};

/****************************************************************/

inline OPValueRef::OPValueRef (OPObjectIn x) : OPObjectRef ()
{ this->NewValue ()->SetObject (x); }

inline OPValueRef::OPValueRef (OPValueIn x) : OPObjectRef (x)
{ }

inline OPValueRef::OPValueRef (OPValue* x) : OPObjectRef (x)
{ }

inline OPValueRef::OPValueRef (Card32 x)
{ this->NewValue ()->SetCard32 (x); }

inline OPValueRef::OPValueRef (Int32 x)
{ this->NewValue ()->SetInt32 (x); }

inline OPValueRef::OPValueRef (Bool x)
{ this->NewValue ()->SetBool (x); }

inline OPValueRef::OPValueRef (Real x)
{ this->NewValue ()->SetReal (x); }

inline OPValueRef& OPValueRef::operator = (OPObjectIn x)
{ this->NewValue ()->SetObject (x); return *this; }

inline OPValueRef& OPValueRef::operator = (OPValueIn x)
{ this->_Set (x); return *this; }

inline OPValueRef& OPValueRef::operator = (OPValue* x)
{ this->_Set (x); return *this; }

inline OPValueRef& OPValueRef::operator = (Card32 x)
{ this->NewValue ()->SetCard32 (x); return *this; }

inline OPValueRef& OPValueRef::operator = (Int32 x)
{ this->NewValue ()->SetInt32 (x); return *this; }

inline OPValueRef& OPValueRef::operator = (Bool x)
{ this->NewValue ()->SetBool (x); return *this; }

inline OPValueRef& OPValueRef::operator = (Real x)
{ this->NewValue ()->SetReal (x); return *this; }

/****************************************************************/

inline OPValueRef::operator Card32 () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnCard32 () : 0; }

inline OPValueRef::operator Int32 () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnInt32 () : 0; }

inline OPValueRef::operator Bool () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnBool () : 0; }

inline OPValueRef::operator Real () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnReal () : 0; }

/****************************************************************/

extern inline Bool
operator == (OPValueIn v, Card32 x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

extern inline Bool
operator == (OPValueIn v, Int32 x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

extern inline Bool
operator == (OPValueIn v, Bool x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

extern inline Bool
operator == (OPValueIn v, Real x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

/****************************************************************/

inline OPValue*
OPValueRef::CheckAndResolve () const
{
	return (OPValue*)((this->object) ? this->object->CheckAndResolve ()
									 : this->ReturnRuntime ()->ResolveNull ());
}

inline OPValue*
OPValueRef::Resolve () const
{
	return (OPValue*)((this->object) ? this->object->Resolve () : 0);
}

inline OPValue*
OPValueRef::operator -> () const
{
	return (OPValue*)(this->CheckAndResolve ());
}

/****************************************************************/

#endif /* _OPaC_VALUES_H_ */
