/*
 *	sd_pipeclient.cxx
 *
 *	The pipe client is the client part of a remote connection;
 *	this is machine dependent.
 *
 *	(C) Copyright 1994-1997, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "sd_interface.h"

#include <OPaC/pipestream.h>
#include <OPaC/new.h>

/****************************************************************/

OP_IMPLEMENTATION (pipestream.h);
OP_IMPLEMENT_CLASS (OPPipeClient);

/****************************************************************/

#define	DEFAULT_TIMEOUT	60.0		//	just a minute

/****************************************************************/

struct OPaC_CPipe
{
	HANDLE				handle;
	Bool				is_connected;
};

/****************************************************************/


OPPipeClient::OPPipeClient ()
{
	this->pipe = NEW (OPaC_CPipe);

	this->pipe->handle       = 0;
	this->pipe->is_connected = FALSE;
	
	this->host_name   = OP_NEW (OPString);
	this->server_name = OP_NEW (OPString);
	this->pipe_name   = OP_NEW (OPString);
	this->timeout     = DEFAULT_TIMEOUT;
}

OPPipeClient::~OPPipeClient ()
{
	if (this->pipe->is_connected) {
		this->Close ();
	}
	
	DELETE (this->pipe);
}

void
OPPipeClient::StoreInstance (OPStorageIn storage)
{
}

void
OPPipeClient::RestoreInstance (OPStorageIn storage)
{
}


Bool
OPPipeClient::SetHostName (OPStringIn name)
{
	return this->host_name->SetString (name);
}


Bool
OPPipeClient::SetServerName (OPStringIn name)
{
	return this->server_name->SetString (name);
}


Bool
OPPipeClient::SetPipeName (OPStringIn name)
{
	return this->pipe_name->SetString (name);
}


Bool
OPPipeClient::SetTimeOut (Real time)
{
	this->timeout = time;
	return TRUE;
}


Bool
OPPipeClient::Request (const void* request, Count request_len, void* answer, Count& answer_len)
{
	return FALSE;
}


void
OPPipeClient::Open ()
{
	if (this->pipe->is_connected) {
		OP_ASSERT (this->pipe->is_connected == FALSE);
	}
	
	OPStringRef name = OP_NEW (OPString);
	DWORD       time = (DWORD)(timeout*1000);
	DWORD       mode = PIPE_READMODE_MESSAGE;

	name->AppendString ("\\\\.\\pipe\\");
	name->AppendString (this->server_name);
	name->AppendString (":");
	name->AppendString (this->pipe_name);
	name->ConvertToLatin ();
	
	while (1) {
		this->pipe->handle = CreateFile (name->AccessLatin (),
										 GENERIC_READ | GENERIC_WRITE,
										 0,
										 NULL,
										 OPEN_EXISTING,
										 0,
										 NULL);
		
		if (this->pipe->handle != INVALID_HANDLE_VALUE) break;
		if (GetLastError () != ERROR_PIPE_BUSY) OP_ASSERT (FALSE);
		
		WaitNamedPipe (name->AccessLatin (), time);
	}
	
	SetNamedPipeHandleState (this->pipe->handle,
							 &mode, NULL, NULL);
	
	this->pipe->is_connected = TRUE;
}


Count
OPPipeClient::Read (void* buffer, Count size)
{
	if (this->pipe->is_connected == FALSE) {
		return 0;
	}
	
	Bool  ok;
	
	DWORD rdsize = (DWORD)(size);
	DWORD time   = (DWORD)(timeout*1000);
	DWORD total  = 0;
	
	for (;;) {
		
		PeekNamedPipe (this->pipe->handle, NULL, 0, NULL, &total, NULL);
		
		if (total) break;
		if (time == 0) return 0;
		
		if (time < 100) {
			Sleep (time);
			time = 0;
		} else {
			Sleep (100);
			time -= 100;
		}
	}
	
	ok = ReadFile (this->pipe->handle, buffer, rdsize, &rdsize, NULL);
	
	if (ok == FALSE) {
		return 0;
	}
	
	return (Count)(rdsize);
}


Count
OPPipeClient::Write (const void* buffer, Count size)
{
	if (this->pipe->is_connected == FALSE) {
		return 0;
	}
	
	Bool ok;
	DWORD wrsize = (DWORD)(size);
	
	ok = WriteFile (this->pipe->handle, buffer, wrsize, &wrsize, NULL);
	
	if (ok == FALSE) {
		return 0;
	}

	return (Count)(wrsize);
}


void
OPPipeClient::Sync ()
{
}


void
OPPipeClient::Close ()
{
	if (this->pipe->is_connected == TRUE) {
		CloseHandle (this->pipe->handle);
		this->pipe->is_connected = FALSE;
	}
}

