/*
 *	gcontnr.cxx
 *
 *	Base OPaC Generic Containter class. This is just an abstract class
 *	implementation. Hash tables and sorted lists inherit its
 *	properties.
 *
 *	(C) Copyright 1994-1997, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/container.h>
#include <OPaC/iterator.h>
#include <OPaC/pubmacro.h>
#include <OPaC/new.h>

/****************************************************************/

OP_ABSTRACT_CLASS (OPGenericContainer);

/****************************************************************/

OP_PUBLISH_FUNCTION (OPGenericContainer, "TotalCount", TotalCount, "Total number of elements", Card32, Card32);

/****************************************************************/

/*
 *	The generic container is a fully abstract class.
 */

OPGenericContainer::OPGenericContainer ()
{
	OP_METHOD (OPGenericContainer, 0);
	
	this->iterators = 0;
}



/*
 *	Let the subclasses handle the destruction, since this
 *	is just a fully abstract class.
 */

OPGenericContainer::~OPGenericContainer ()
{
	OP_METHOD (~OPGenericContainer, 0);
}



void
OPGenericContainer::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	storage << this->iterators;
	storage << OPStorageSync::mark;
	
	super::StoreInstance (storage);
}

void
OPGenericContainer::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	storage >> this->iterators;
	storage >> OPStorageSync::mark;
	
	super::RestoreInstance (storage);
}


void
OPGenericContainer::Delete ()
{
	OP_METHOD (Delete, 0);
	
	if (this->iterators.IsValid ()) {
		this->iterators->Delete ();
		this->iterators = 0;
	}
	
	super::Delete ();
}

void
OPGenericContainer::InsertIterator (OPIteratorIn iterator)
{
	OP_METHOD (InsertIterator, 1);
	OP_ARG (1, OPIteratorIn, iterator);
	
	if (this->iterators.IsValid () == FALSE) {
		this->iterators = OP_NEW (OPList);
	}
	
	this->iterators->Insert (iterator);
	iterator->SetSource (this);
}

void
OPGenericContainer::RemoveIterator (OPIteratorIn iterator)
{
	OP_METHOD (RemoveIterator, 1);
	OP_ARG (1, OPIteratorIn, iterator);
	
	OP_ASSERT (this->iterators.IsValid ());
	
	this->iterators->Remove (iterator);
}


OPIteratorRef
OPGenericContainer::NewIterator ()
{
	OP_METHOD (NewIterator, 0);
	
	return OPIteratorRef (0);
}

