/*
 *	listit.cxx
 *
 *	The List needs a dedicated Iterator class.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/container.h>
#include <OPaC/listit.h>
#include <OPaC/new.h>

OP_IMPLEMENT_CLASS (OPListIterator);

OPListIterator::OPListIterator ()
{
	this->current = 0;
}

OPListIterator::~OPListIterator ()
{
}

void
OPListIterator::StoreInstance (OPStorageIn storage)
{
	super::StoreInstance (storage);
	storage << OPStorageSync::mark;
}

void
OPListIterator::RestoreInstance (OPStorageIn storage)
{
	super::RestoreInstance (storage);
	storage >> OPStorageSync::mark;
}


void
OPListIterator::NotifyInsert (OPList::Element* elem)
{
}


void
OPListIterator::NotifyRemove (OPList::Element* elem)
{
	if (this->current == elem) {
		if (elem->next) {
			this->current = elem->next;
		} else if (elem->prev) {
			this->current = elem->prev;
		} else {
			this->current = 0;
		}
		this->did_change = TRUE;
	}
}


void
OPListIterator::Delete ()
{
	super::Delete ();
}

Bool
OPListIterator::First ()
{
	OPListRef        list  = this->source;
	OPList::Element* first = list->head;
	
	if (this->current == first) {
		return TRUE;
	}
	
	this->did_change = TRUE;
	this->current    = first;
	this->is_valid   = TRUE;
	
	return TRUE;
}

Bool
OPListIterator::Last ()
{
	OPListRef        list = this->source;
	OPList::Element* last = list->tail;
	
	if (this->current == last) {
		return TRUE;
	}
	
	this->did_change = TRUE;
	this->current    = last;
	this->is_valid   = TRUE;
	
	return TRUE;
}

Bool
OPListIterator::Next (Count move)
{
	if ( (this->current)
	  && (this->current->next) ) {
		this->current    = this->current->next;
		this->did_change = TRUE;
		this->is_valid   = TRUE;
		return TRUE;
	}
	
	this->is_valid = FALSE;
	
	return FALSE;
}

Bool
OPListIterator::Prev (Count move)
{
	if ( (this->current)
	  && (this->current->prev) ) {
		this->current    = this->current->prev;
		this->did_change = TRUE;
		this->is_valid   = TRUE;
		return TRUE;
	}
	
	this->is_valid = FALSE;
	
	return FALSE;
}

Bool
OPListIterator::Get (OPObjectOut object, Count index)
{
	this->did_change = FALSE;
	
	if ( (index == 0)
	  && (this->current)
	  && (this->is_valid) ) {
		object = this->current->data;
		return TRUE;
	}
	
	return FALSE;
}
