/*
 *	buildedit.h
 *
 *	
 *
 *	(C) Copyright 1997-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _BUILDEDIT_H_
#define _BUILDEDIT_H_

#include <OPaC/dbscrlist.h>
#include <OPaC/object.h>
#include <OPaC/objectpane.h>
#include <OPaC/opexport.h>
#include <OPaC/rect.h>

/****************************************************************/

APP_EXPORT_CLASS (OPBuildEdit);

/****************************************************************/

APP_DEFINE_REF (OPBuildEdit, OPObject);

/****************************************************************/

class OPBuildEdit : public OPObject
{
private:
	OPWindowRef			window;
	OPObjectRef			object;
	OPViewRef			attached_events;
	OPHashTableRef		reporters;
	
protected:
	OPBuildEdit ();
	virtual ~ OPBuildEdit ();
	
	virtual void Cleanup ();
	
	static void DoAddReporter (OPObjectIn signal, OPDataReportIn reporter);
	
public:
	virtual Bool EditProperties (OPObjectIn object);
	virtual Bool Update ();
	virtual Bool EditOK ();
	
	virtual void Delete (); /* OVERRIDDEN */
	virtual void AddReporter (OPDataReportIn reporter);
	
	const OPWindowRef& ReturnWindow () const	{ return this->window; }
	const OPObjectRef& ReturnObject () const	{ return this->object; }
	
	static OPSimpleType FindSimpleType (OPObjectIn edit, OPTypeInfoIn type, OPAtomIn name);
	
	static void EditAnyProperty (OPObjectIn me, OPObjectIn edit, OPTypeInfoIn type, OPAtomIn name, OPViewOut view);
	static void EditTextProperty (OPObjectIn me, OPObjectIn edit, OPTypeInfoIn type, OPAtomIn name, OPViewOut view);
	static void EditTextTagProperty (OPObjectIn me, OPObjectIn edit, OPTypeInfoIn type, OPAtomIn name, OPViewOut view);
	static void EditNumericPropery (OPObjectIn me, OPObjectIn edit, OPTypeInfoIn type, OPAtomIn name, OPViewOut view);
	static void EditBoolProperty (OPObjectIn me, OPObjectIn edit, OPTypeInfoIn type, OPAtomIn name, OPViewOut view);
	static void EditViewProperty (OPObjectIn me, OPObjectIn edit, OPTypeInfoIn type, OPAtomIn name, OPViewOut view);
	static void EditObjectProperty (OPObjectIn me, OPObjectIn edit, OPTypeInfoIn type, OPAtomIn name, OPViewOut view);
	static void EditBitsetProperty (OPObjectIn me, OPObjectIn edit, OPTypeInfoIn type, OPAtomIn name, OPViewOut view);
	
	static Bool HasAttachedTarget (OPObjectIn edit, OPAtomIn find);
	static Bool HasAttachedDataTarget (OPObjectIn edit, OPAtomIn find, OPAtomIn notifier);
	
	static void MakeEventList (OPDBScrollListIn list, OPObjectIn edit);
	static void MakeTargetList (OPDBScrollListIn list, OPObjectIn edit);
	static void MakeDataSourceList (OPDBScrollListIn list, OPObjectIn edit);
	static void MakeDataTargetList (OPDBScrollListIn list, OPObjectIn edit);

	static void MakeAttachedList (OPViewIn parent, OPObjectIn edit, OPStringIn title, OPViewOut view);
	
	APP_DEFINE_CLASS (OPBuildEdit, OPObject);
};

/****************************************************************/

#endif /* _BUILDEDIT_H_ */
