/*
 *	connectlv.cxx
 *
 *	Special view which paints itself either as one single rectangle
 *	(the destination of a connection) or as a black box (source and
 *	vertical/horizontal connecting lines).
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "connectline.h"

#include <OPaC/grail.h>
#include <OPaC/lookman.h>
#include <OPaC/new.h>
#include <OPaC/pubmacro.h>
#include <OPaC/targetmacro.h>
#include <OPaC/window.h>

/****************************************************************/

APP_IMPLEMENT_CLASS (OPConnectLineView);

OPConnectLineView::OPConnectLineView ()
{
	this->draw_just_rect = FALSE;
}

OPConnectLineView::~OPConnectLineView ()
{
}

void
OPConnectLineView::Draw (OPGrailPortIn port, OPRectIn rect, OPDrawOp what)
{
	OP_METHOD (Draw, 3);
	OP_ARG (1, OPGrailPortIn, port);
	OP_ARG (2, OPRectIn, rect);
	OP_ARG (3, OPDrawOp, what);
	
	const OPAdornerRef& adorner = port->ReturnAdorner ();
	
	OPRect box (this->window->ReturnBox ());
	box.SetOrigin (0, 0);
	
	if (this->draw_just_rect) {
		adorner->DrawRect (box, OP_COLOR_VIEW_CONNECT, 0);
		adorner->DrawRect (box, OP_COLOR_VIEW_CONNECT, 1);
	} else {
		adorner->FillRect (box, OP_COLOR_VIEW_CONNECT, 0);
	}
}

void
OPConnectLineView::DrawJustRect (Bool yes)
{
	this->draw_just_rect = yes;
}
